import datetime
import logging

from upswingutil.db import Firestore, Mongodb
from upswingutil.schema import ResponseDict, CheckinReservationModel
from upswingutil.db.model import ReservationModelV2, Status


def check_in_reservation(db: Firestore, agent, orgId: str, data: CheckinReservationModel) -> ResponseDict:
    response = ResponseDict()
    mongo = Mongodb(orgId)
    reservation = ReservationModelV2(
        _id=data.reservationId,
        orgId=orgId,
        agent=agent,
        datesAndDuration={
            'createdDate': datetime.datetime.now(),
            'confirmedDate': datetime.datetime.now(),
            'arrivalDate': data.arrivalDate,
            'departureDate': data.arrivalDate,
            'cancelledDate': '',
        },
        status=Status.ARRIVED.value,
        property={
            'hotelId': data.hotelId,
            'roomId': data.roomId,
            'roomName': data.roomId,
            'categoryName': '',
            'numberOfRooms': 1,
        },
        auraRecordUpdatedOn=datetime.datetime.now()
    )
    result = mongo.get_collection(mongo.RESERVATION_COLLECTION)\
        .update_one(
            {'_id': data.reservationId},
            {'$set': reservation.to_mongo()},
            upsert=True
        )
    if result.acknowledged:
        db.get_collection(f'Organizations/{orgId}/reservations').document(data.reservationId).set(reservation.to_mongo(), merge=True)
        response.status = True
    return response



from .auth import validate_key, get_key
from .variables import NAME, NAME_ON_PREM
# from .reservation import ReservationSync
from .reservation_v2 import ReservationSync
from .check_in import check_in_reservation



import logging
from upswingutil.db import Firestore
from upswingutil.db.model import ReservationModelV2
from upswingutil.schema import GuestType


def store_reservation_to_alvie(record: ReservationModelV2):
    try:
        alvieDB = Firestore(app='alvie')
        record = record.to_mongo()
        for guest in record.get('guestInfo').get('guest_list'):
            guest = dict(guest)
            # to store reservation only in the profile of the primary guest
            # incase if we have to make it store for all the guests, then remove this condition
            if guest.get('primary'):
                docs = alvieDB.get_collection(f'Organizations/{record.get("orgId")}/users') \
                    .where("pmsGuestProfile", "==", guest.get('guest')).stream()
                for doc in docs:
                    alvieDB.get_collection(f'Organizations/{record.get("orgId")}/users/{doc.id}/reservations') \
                        .document(str(record.get('_id'))).set(record, merge=True)
                    logging.info(
                        f"reservation {record.get('orgId')} : {record.get('_id')} added to alvie profile {doc.id}")
    except Exception as e:
        logging.error(f'Error while storing reservation {record.get("orgId")} : {record.get("_id")} to firestore')
        logging.error(e)


def store_reservation_to_alvie_v2(record: ReservationModelV2):
    """
    Store the reservation into firestore only if the user's profile available in firestore to link with
    Args:
        record: reservation record version 2
    Author: Harsh Mathur
    """
    try:
        alvieDB = Firestore(app='alvie')
        record = record.to_mongo()
        for guest in record.get('guestInfo').get('guest_list'):
            guest = dict(guest)
            guestUID = guest.get('appGuestUID')
            if guestUID and guestUID is not '':
                record['guest_app_data'][guestUID] = {
                    'appAccess': True,
                    'primary': guest.get('primary'),
                    'doorlockAccess': True,
                    'grmsAccess': True,
                    'overallRating': 0,
                    'overallReview': '',
                    'guestType': GuestType.PRIMARY if guest.get('primary') else GuestType.CO_TRAVELLER
                }
        alvieDB.get_collection(f'Organizations/{record.get("orgId")}/reservations') \
            .document(str(record.get('_id'))).set(record, merge=True)
        logging.info(f"reservation {record.get('orgId')} : {record.get('_id')} added to firestore")
    except Exception as e:
        logging.error(f'Error while storing reservation {record.orgId} : {record.id} to firestore')
        logging.error(e)



import logging

from upswingutil.db import Firestore
from upswingutil.schema import ResponseDict, CheckinReservationModel


def check_in_reservation(db: Firestore, orgId: str, data: CheckinReservationModel) -> ResponseDict:
    response = ResponseDict()
    pass



from enum import Enum
from typing import List, Optional
from pydantic import BaseModel


class GuestType(str, Enum):
    PRIMARY = 'Primary'
    CO_TRAVELLER = 'Co-traveller'
    SPOUSE = 'Spouse'
    CHILDREN = 'Children'


class CheckInGuestModel(BaseModel):
    name: str = ""
    guestEmail: str = ""
    guestPMSId: str = ""
    appGuestUID: str = ""
    guestType: GuestType = GuestType.CO_TRAVELLER
    primary: bool = ""
    appAccess: bool = ""
    doorlockAccess: bool = ""
    grmsAccess: bool = ""
    doorlockKey: Optional[str] = ""
    doorlockSpecialAccess: Optional[str] = ""



class CheckinReservationModel(BaseModel):
    reservationId: str
    hotelId: str
    arrivalDate: str = ""
    departureDate: str = ""
    roomId: str = ""
    checkin_type: str = ""
    doorlockLinked: bool = False
    grmsLinked: bool = False
    appLinked: bool = False
    guest_list: List[CheckInGuestModel] = []




import logging
import upswingutil as ul
from firebase_admin import firestore, get_app, storage


class Firestore:
    org_collection = 'Organizations'
    sub_collection_properties = 'properties'
    credentials = 'credentials'

    def __init__(self, app=None, project_name=ul.G_CLOUD_PROJECT):
        self.firestore_db = firestore.client(app=get_app(app)) if app else firestore.client()
        self.firestore_storage = storage.bucket(name=f'{project_name}.appspot.com',
                                                app=get_app(app)) if app else storage.bucket(
            name=f'{project_name}.appspot.com')

    def get_collection(self, name):
        return self.firestore_db.collection(name)

    def get_collection_docs(self, name):
        return self.firestore_db.collection(name).stream()

    def get_ref_collection(self, name):
        return self.firestore_db.collection(name)

    def get_ref_document(self, name, doc):
        return self.firestore_db.collection(name).document(doc).get()

    def delete_doc(self, collection: str, doc: str):
        __ref__ = self.firestore_db.collection(collection).document(doc)
        __ref__.delete()
        logging.debug(f"Document deleted successfully")

    def write_doc(self, collection: str, doc: str or None, data: dict):
        __ref__ = self.firestore_db.collection(collection).document(doc)
        __ref__.set(data, merge=True)
        logging.debug(f"Document written successfully")



    def write_doc_auto_id(self, collection: str, data: dict):
        __ref__ = self.firestore_db.collection(collection)
        ref = __ref__.add(data)
        logging.debug(f"Document written successfully")
        return ref      #this function will return tuple

    def update_doc(self, collection: str, doc: str, data: dict):
        __ref__ = self.firestore_db.collection(collection).document(doc)
        __ref__.update(data)
        logging.debug(f"Document Updated successfully")

    def increment_doc_value(self, collection: str, doc: str, data: str, incr: int):
        __ref__ = self.firestore_db.collection(collection).document(doc)
        __ref__.update({
            data: firestore.firestore.Increment(incr)
        })

    def upload_document_firestorage(self, path: str, contentType: str, imgData: bytes, public: bool = False):
        blob_ref = self.firestore_storage.blob(path)
        blob_ref.upload_from_string(imgData, content_type=contentType)
        if public:
            blob_ref.make_public()
        return blob_ref.public_url

    def delete_document_firestorage(self, path: str):
        blob_ref = self.firestore_storage.blob(path)
        if (blob_ref.exists()):
            blob_ref.delete()
        return True

    def duplicate_document(self, from_path: str, to_path: str, fileName: str):
        data = self.firestore_db.collection(from_path).document(fileName).get()
        if data.exists:
            self.write_doc(to_path, fileName, data.to_dict())

    def duplicate_collection(self, from_path: str, to_path: str):
        data = self.get_collection_docs(from_path)
        for doc in data:
            self.write_doc(to_path, doc.id, doc.to_dict())
